@@ -1,4 +1,5 @@
 """ Common functions for package biobb_cmip.cmip """
+import os
 import re
 import json
 from pathlib import Path
@@ -10,14 +11,14 @@ import logging
 import biobb_common.tools.file_utils as fu
 
 
-def get_grid(cmip_log_path: Union[str, Path]) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
+def get_grid(cmip_log_path: Union[str, Path], external: bool = False) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
     with open(cmip_log_path) as log_file:
         first_line = log_file.readline().strip()
     if first_line.startswith("titleParam"):
-        return _get_grid_from_key_value(cmip_log_path)
+        return _get_grid_from_key_value(cmip_log_path, external)
     elif first_line.startswith("{"):
         return _get_grid_from_box_file(cmip_log_path)
-    return _get_grid_from_text(cmip_log_path)
+    return _get_grid_from_text(cmip_log_path, external)
 
 
 def _get_grid_from_box_file(cmip_box_path: Union[str, Path]) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
@@ -28,15 +29,18 @@ def _get_grid_from_box_file(cmip_box_path: Union[str, Path]) -> Tuple[Tuple[floa
     return origin, size, grid_dict['params']
 
 
-def _get_grid_from_text(cmip_log_path: Union[str, Path]) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
+def _get_grid_from_text(cmip_log_path: Union[str, Path], external: bool = False) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
     origin = None
     size = None
     grid_params = {"CEN": None, "DIM": None, "INT": None}
+    grid_locators_list = ["AUTOMATIC GRID", "MANUAL GRID"]
+    if external:
+        grid_locators_list = ["AUTOMATIC OUTER GRID", "MANUAL OUTER GRID"]
 
     with open(cmip_log_path) as log_file:
         inside_automatic_grid = False
         for line in log_file:
-            if line.strip() in ["AUTOMATIC GRID", "MANUAL GRID"]:
+            if line.strip() in grid_locators_list:
                 inside_automatic_grid = True
             if inside_automatic_grid:
                 origin_match = re.match(r"Grid origin:\s+([-+]?(?:\d*\.\d+|\d+))\s+([-+]?(?:\d*\.\d+|\d+))\s+([-+]?(?:\d*\.\d+|\d+))", line.strip())
@@ -59,15 +63,18 @@ def _get_grid_from_text(cmip_log_path: Union[str, Path]) -> Tuple[Tuple[float, f
     return origin, size, grid_params
 
 
-def _get_grid_from_key_value(cmip_log_path: Union[str, Path]) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
+def _get_grid_from_key_value(cmip_log_path: Union[str, Path], external: bool = False) -> Tuple[Tuple[float, float, float], Tuple[float, float, float], Mapping[str, Tuple[float, float, float]]]:
     origin = None
     size = None
     grid_params = {"CEN": None, "DIM": None, "INT": None}
+    grid_locators_list = ["AUTOMATIC GRID", "MANUAL GRID"]
+    if external:
+        grid_locators_list = ["AUTOMATIC OUTER GRID", "MANUAL OUTER GRID"]
 
     with open(cmip_log_path) as log_file:
         inside_automatic_grid = False
         for line in log_file:
-            if line.strip() in ["AUTOMATIC GRID", "MANUAL GRID"]:
+            if line.strip() in grid_locators_list:
                 inside_automatic_grid = True
             if inside_automatic_grid:
                 origin_match = re.match(r"origin=\s+([-+]?(?:\d*\.\d+|\d+))\s*,\s*([-+]?(?:\d*\.\d+|\d+))\s*,\s*([-+]?(?:\d*\.\d+|\d+))", line.strip())
@@ -134,6 +141,7 @@ def get_topology_cmip_elements_canonical(input_topology_filename: str) -> List:
                     continue
                 nt.write(line)
         u = mda.Universe(top_file, topology_format="ITP")
+        os.unlink(top_file)
     else:
         u = mda.Universe(input_topology_filename)
     # mda_charges = [round(val, 4) for val in u.atoms.charges]
@@ -161,12 +169,14 @@ def get_topology_charges(input_topology_filename: str) -> List:
         with open(input_topology_filename) as tf:
             top_lines = tf.readlines()
         top_file = create_unique_file_path(parent_dir=Path(input_topology_filename).parent.resolve(), extension='.top')
+
         with open(top_file, 'w') as nt:
             for line in top_lines:
                 if re.search(r"\.ff.*\.itp", line):
                     continue
                 nt.write(line)
         u = mda.Universe(top_file, topology_format="ITP")
+        os.unlink(top_file)
     else:
         u = mda.Universe(input_topology_filename)
     return [round(val, 4) for val in u.atoms.charges]
@@ -295,7 +305,7 @@ def probe_params_grid(probe_id: int = 0, readgrid: int = 2, pbfocus: int = 1, pe
     return grid_dict
 
 
-def params_grid(grid_type: str = 'cmip', readgrid: int = 0, perfill: float = 0.8,
+def params_grid(grid_type: str, readgrid: int = 0, perfill: float = 0.8,
                 grid_int: Sequence[float] = (0.5, 0.5, 0.5),
                 grid_dim: Sequence[float] = (64, 64, 64),
                 grid_cen: Sequence[float] = (0.0, 0.0, 0.0)) -> Dict[str, str]:
@@ -308,7 +318,7 @@ def params_grid(grid_type: str = 'cmip', readgrid: int = 0, perfill: float = 0.8
     grid_dict = {}
     grid_dict[f"readgrid"] = f"{readgrid}"
 
-    if grid_type in ['interaction', 'mip', 'energy', 'docking']:
+    if grid_type in ['pb_interaction_energy', 'mip_pos', 'mip_neu', 'mip_neg', 'docking']:
         grid_dict['grid_cen'] = f"CENX={grid_cen[0]},CENY={grid_cen[1]},CENZ={grid_cen[2]}"
         grid_dict['grid_dim'] = f"DIMX={grid_dim[0]},DIMY={grid_dim[1]},DIMZ={grid_dim[2]}"
         grid_dict['grid_int'] = f"INTX={grid_int[0]},INTY={grid_int[1]},INTZ={grid_int[2]}"
@@ -323,8 +333,9 @@ def params_preset(execution_type: str) -> Dict[str, str]:
     params_dict = {}
     grid_dict = {}
     probe_grid_dict = {}
+    execution_type = execution_type.strip()
     if execution_type == 'titration':
-        grid_dict = params_grid(grid_type='titration', readgrid=2, perfill=0.8, grid_int=(0.5, 0.5, 0.5) )
+        grid_dict = params_grid(grid_type=execution_type, readgrid=2, perfill=0.8, grid_int=(0.5, 0.5, 0.5) )
         params_dict = {
             'title': 'Titration',
             'tipcalc': 1,
@@ -336,8 +347,8 @@ def params_preset(execution_type: str) -> Dict[str, str]:
             'titration': 1, 'inifoc': 2, 'cutfoc': -0.5, 'focus': 1, 'ninter': 10, 'clhost': 1, 'titcut': 20.,
             'titwat': 10, 'titip': 10, 'titim': 10
         }
-    elif execution_type.strip() == 'mip_pos':
-        grid_dict = params_grid(grid_type='mip', readgrid=2)
+    elif execution_type == 'mip_pos':
+        grid_dict = params_grid(grid_type=execution_type, readgrid=2)
         params_dict = {
             'title': 'MIP positive probe',
             'tipcalc': 0,
@@ -351,8 +362,8 @@ def params_preset(execution_type: str) -> Dict[str, str]:
             'carmip': 1,
             'tipatmip' : "'OW'"
         }
-    elif execution_type.strip() == 'mip_neu':
-        grid_dict = params_grid(grid_type='mip', readgrid=2)
+    elif execution_type == 'mip_neu':
+        grid_dict = params_grid(grid_type=execution_type, readgrid=2)
         params_dict = {
             'title': 'MIP neutral probe',
             'tipcalc': 0,
@@ -366,8 +377,8 @@ def params_preset(execution_type: str) -> Dict[str, str]:
             'carmip': 0,
             'tipatmip' : "'OW'"
         }
-    elif execution_type.strip() == 'mip_neg':
-        grid_dict = params_grid(grid_type='mip', readgrid=2)
+    elif execution_type == 'mip_neg':
+        grid_dict = params_grid(grid_type=execution_type, readgrid=2)
         params_dict = {
             'title': 'MIP negative probe',
             'tipcalc': 0,
@@ -384,7 +395,7 @@ def params_preset(execution_type: str) -> Dict[str, str]:
 #TODO 'carmip': 1,
     # wat: tipcalc: 1 + titration: 'inifoc': 2, 'cutfoc': -0.5, 'focus': 1, 'ninter': 10,
     elif execution_type == 'solvation':
-        grid_dict = params_grid(grid_type='solvation', readgrid=2, perfill=0.2,
+        grid_dict = params_grid(grid_type=execution_type, readgrid=2, perfill=0.2,
                                 grid_int=(0.5, 0.5, 0.5))
         params_dict = {
             'title': 'Solvation & MEP',
@@ -397,10 +408,12 @@ def params_preset(execution_type: str) -> Dict[str, str]:
             'novdwgrid': 1, 'solvenergy': 1, 'dielc': 1, 'dielsol': 80
         }
 
-    elif execution_type == 'energy':
-        grid_dict = params_grid(grid_type='energy', readgrid= 2)
+    elif execution_type == 'pb_interaction_energy':
+        grid_dict = params_grid(grid_type=execution_type, readgrid= 2)
         probe_grid_dict = probe_params_grid(probe_id= 0, readgrid= 2, pbfocus= 1, perfill= 0.6,
                                             grid_int=(1.5, 1.5, 1.5))
+
+        # TODO Check for external box file or parameters
         params_dict = {
             'title': 'Docking Interaction energy calculation. PB electrostatics',
             'tipcalc': 3,
@@ -412,7 +425,7 @@ def params_preset(execution_type: str) -> Dict[str, str]:
         }
 
     elif execution_type == 'docking':
-        grid_dict = params_grid(grid_type='docking', readgrid= 2)
+        grid_dict = params_grid(grid_type=execution_type, readgrid= 2)
 
         params_dict = {
             'title': 'Docking Mehler Solmajer dielectric',
@@ -493,7 +506,7 @@ def write_params_file(output_params_path: str, params_dict: Mapping[str, str]) -
         output_params_file.write(f"{params_dict.pop('title', 'Untitled')}\n")
         output_params_file.write(f"&cntrl\n")
         for params_key, params_value in params_dict.items():
-            if params_key in ['grid_int', 'grid_cen', 'grid_dim']:
+            if params_key in ['grid_int', 'grid_cen', 'grid_dim', 'grid_int0', 'grid_cen0', 'grid_dim0']:
                 output_params_file.write(f" {params_value}\n")
             else:
                 output_params_file.write(f" {params_key} = {params_value}\n")
