@@ -66,6 +66,7 @@ class PrepareStructure(BiobbObject):
 
         # Call parent class constructor
         super().__init__(properties)
+        self.locals_var_dict = locals().copy()
 
         # Input/Output files
         self.io_dict = {
@@ -73,9 +74,9 @@ class PrepareStructure(BiobbObject):
             "out": {"output_cmip_pdb_path": output_cmip_pdb_path}
         }
 
-
         # Check the properties
         self.check_properties(properties)
+        self.check_arguments()
 
     @launchlogger
     def launch(self) -> int:
@@ -83,8 +84,6 @@ class PrepareStructure(BiobbObject):
         # Setup Biobb
         if self.check_restart(): return 0
 
-        #######################################
-
         # Dani's method
         if self.io_dict['in']['input_topology_path']:
             top_file = self.io_dict['in']['input_topology_path']
@@ -113,9 +112,13 @@ class PrepareStructure(BiobbObject):
                        elements_list)
 
         ###################################
-        # Remove temporal files
+
+        # remove temporary folder(s)
+        self.tmp_files.extend([top_dir])
         self.remove_tmp_files()
 
+        self.check_arguments(output_files_created=True, raise_exception=False)
+
         return 0
 
 
@@ -145,8 +148,10 @@ def main():
     properties = settings.ConfReader(config=config).get_prop_dic()
 
     # Specific call of each building block
-    prepare_structure(input_pdb_path=args.input_pdb_path, output_cmip_pdb_path=args.output_cmip_pdb_path,
-                      input_topology_path=args.input_topology_path, properties=properties)
+    prepare_structure(input_pdb_path=args.input_pdb_path,
+                      output_cmip_pdb_path=args.output_cmip_pdb_path,
+                      input_topology_path=args.input_topology_path,
+                      properties=properties)
 
 
 if __name__ == '__main__':
